<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of XcruderActiveRecord
 *
 * @author nlac
 */
class XcruderActiveRecord extends CActiveRecord {
	
	public static function safeArray($a) {
		if (!$a)
			return array();
		if (is_array($a))
			return $a;
		return array($a);
	}
	
/**
 * Finds a record by pk
 * 
 */
	public static function xpFindByPk($modelPk, $modelClass, $bWithRelated) {
		
		if (!@$modelPk || !@$modelClass)
			return null;
		
		//$model = CActiveRecord::model($modelClass);
		$model = $modelClass::model();

		if ($bWithRelated) {
			$model = $model->with( $model->relations() );
		}

		return (is_array($modelPk) ? 
			$model->findByAttributes( $modelPk )
						: 
			$model->findByPk( $modelPk ) );
	}

/**
 * Returns a record with attributes
 * 
 */
	public static function getSearchModel($modelClass, $attributes) {
		if (!@$modelClass)
		  return null;

		$model = new $modelClass('search');
		$model->unsetAttributes();

		if (@$attributes)
			$model->attributes = $attributes;
		
		return $model;
	}

/**
 * Returns a record with attributes filled by the request
 * 
 */
	public static function getModelFromRequestSearch($key_modelClass = "modelClass") {
		$modelClass = @$_GET[$key_modelClass];
		if (!@$modelClass)
			return null;
		return self::getSearchModel($modelClass, @$_GET[$modelClass]);
	}

/**
 * Returns a record by pk from the request
 * 
 */
	public static function getModelFromRequestPk($key_modelClass = 'modelClass', $key_pk = 'modelPk') {
		$modelClass = @$_POST[$key_modelClass] ? $_POST[$key_modelClass] : @$_GET[$key_modelClass];
		$modelPk = @$_POST[$key_pk] ? $_POST[$key_pk] : @$_GET[$key_pk];
		$modelPk = @json_decode($modelPk , true);
		$model = self::xpFindByPk($modelPk, $modelClass, true);	
		return $model;
	}

/**
 * Applies a filter from $_GET on the modelArray
 * 
 */
	public static function filterModelArray($modelArray, $modelClass) {
		if (!$modelArray || count($modelArray)==0)
			return null;

		$filterAttributes = @$_GET[$modelClass];
		if (!$filterAttributes)
			return $modelArray;
		
		$keys = array_keys($filterAttributes);
		
		$res = array();
		foreach($modelArray as $rec) {
			$bok = true;
			foreach($filterAttributes as $key=>$val)
				if ($val !== '' && strstr($rec[$key], $val) == false) {
					$bok = false;
					break;
				}
			if (!$bok)
				continue;

			$res[] = $rec;
		}
	
		return $res;
	}
	
	
/**
 * $column : element of $model->tableSchema->columns
 * 'pk', relation object(if fk) or ''
 * 
 * If array, there must be an fkselector grid to be shown
 * 
 **/
	public static function getColumnType($model, $relations, $column) {

		foreach($relations as $name=>&$rel) {
			
			//$column is a foreign key
			if ($column->name == @$rel[2]) {
				return array('fk', $name, $rel);
			}
		
			//if ($column->isPrimaryKey && $rel[0]=='CManyManyRelation') {
			//	return array('pk fk', $name, $rel);
			//}
			
		}

		if ($column->isPrimaryKey)
			return 'pk';

		return '';
	}
	
	
	public static function getNewRecord($modelClass) {
		if (!@$modelClass)
			return null;
		
		$model = new $modelClass();
		
		return $model;
	}

	
/**
 * Used as CGridView initialization:
 * ...
 * 'dataProvider' => XcruderActiveRecord::getSearchDataProvider(...)
 * ...
 * 
 * 
 */
	public static function getSearchDataProvider($model, $opt = array()) {
		if (!$model)
			return null;

		$criteria = new CDbCriteria;
		$columns = $model->tableSchema->columns;
		foreach($model->attributes as $key=>$val) {
			$criteria->compare($key, $val, preg_match('/char|enum|set|text/i', $columns[$key]->dbType));
		}

		$pkColumns = $model->tableSchema->primaryKey;
		
		if (@$opt["excludedRows"][0] && $pkColumns) {

			$pkColumns = XcruderActiveRecord::safeArray($pkColumns);

			$cond = '';
			$recs = &$opt["excludedRows"];
			foreach($recs as $rec) {

				$pattern = " NOT (";
				foreach($pkColumns as $colName) {
					$pattern .= "$colName = '" . $rec->getAttribute($colName) . "' AND";
				}
				$pattern = preg_replace('/AND$/', '', $pattern);

				$cond .= $pattern . ') AND';				
			}

			$cond = preg_replace('/AND$/', '', $cond);
			$criteria->addCondition($cond);

		}//if

		return new CActiveDataProvider(get_class($model), array(
			'criteria' => $criteria,
			'pagination'=>array(
				'pageSize'=> Yii::app()->params['pageSize']
			)
		));

	}
	
	
	public static function getPkColumnNames($columns) {

	  $pks = array();
	  foreach($columns as $col)
		if($col->isPrimaryKey)
		  $pks[] = $col;
	  return $pks;
	}
	
	public static function getPkFromCheckbox($id, $columns) {

	  $pks = self::getPkColumnNames($columns);
	  if (count($pks) <= 1)
		return $id;

	  $ids = preg_split('/,/', $id);
	  $i = 0;
	  $idArray = array();
	  foreach($pks as $col)
		  $idArray[$col->name] = $ids[$i++];

	  return $idArray;
	}
	
	public static function getPairRelationKey($relation, $pairClass, $pairRelations) {
		if (!$relation || !$pairClass || !$pairRelations || $relation[1] != $pairClass)
			return null;
		foreach($pairRelations as $key=>$rel) {
			if ($rel[2] == $relation[2])
				return $key;
		}
		return null;
	}


}

?>
